@@ -1,13 +1,12 @@
 //! Register specifiers (enumerations).
 
-use core::convert::TryFrom;
-
 /// Socket status.
 ///
 /// This is used with the [`sn_sr`] method.
 ///
 /// [`sn_sr`]: crate::Registers::sn_sr
 #[derive(Copy, Clone, Eq, PartialEq, Debug, PartialOrd, Ord, Hash)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 #[repr(u8)]
 pub enum SocketStatus {
     /// Socket closed, this is the reset state of all sockets.
@@ -132,7 +131,7 @@ pub enum SocketStatus {
     /// This state is set when the socket protocol is [`Macraw`], and a [`Open`]
     /// command is sent.
     ///
-    /// [socket 0]: crate::Socket::Socket0
+    /// [socket 0]: crate::Sn::Sn0
     /// [`Macraw`]: Protocol::Macraw
     /// [`Open`]: SocketCommand::Open
     Macraw = 0x42,
@@ -185,6 +184,7 @@ impl Default for SocketStatus {
 /// [`sn_ir`]: crate::Registers::sn_ir
 /// [`sn_sr`]: crate::Registers::sn_sr
 #[derive(Copy, Clone, Eq, PartialEq, Debug, PartialOrd, Ord, Hash)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 #[repr(u8)]
 pub enum SocketCommand {
     /// The command register clears to this state once a command has been
@@ -334,6 +334,24 @@ impl From<SocketCommand> for u8 {
         val as u8
     }
 }
+impl TryFrom<u8> for SocketCommand {
+    type Error = u8;
+    fn try_from(val: u8) -> Result<Self, u8> {
+        match val {
+            x if x == Self::Accepted as u8 => Ok(Self::Accepted),
+            x if x == Self::Open as u8 => Ok(Self::Open),
+            x if x == Self::Listen as u8 => Ok(Self::Listen),
+            x if x == Self::Connect as u8 => Ok(Self::Connect),
+            x if x == Self::Disconnect as u8 => Ok(Self::Disconnect),
+            x if x == Self::Close as u8 => Ok(Self::Close),
+            x if x == Self::Send as u8 => Ok(Self::Send),
+            x if x == Self::SendMac as u8 => Ok(Self::SendMac),
+            x if x == Self::SendKeep as u8 => Ok(Self::SendKeep),
+            x if x == Self::Recv as u8 => Ok(Self::Recv),
+            _ => Err(val),
+        }
+    }
+}
 
 /// Socket protocol.
 ///
@@ -342,6 +360,7 @@ impl From<SocketCommand> for u8 {
 /// [`SocketMode::protocol`]: crate::SocketMode::protocol
 /// [`sn_mr`]: crate::Registers::sn_mr
 #[derive(Copy, Clone, Eq, PartialEq, Debug, PartialOrd, Ord, Hash)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 #[repr(u8)]
 pub enum Protocol {
     /// Closed.
@@ -354,22 +373,27 @@ pub enum Protocol {
     ///
     /// MACRAW mode can only be used with [socket 0].
     ///
-    /// [socket 0]: crate::Socket::Socket0
+    /// [socket 0]: crate::Sn::Sn0
     Macraw = 0b0100,
 }
-impl From<Protocol> for u8 {
-    fn from(val: Protocol) -> u8 {
-        val as u8
-    }
-}
-impl Default for Protocol {
-    fn default() -> Protocol {
-        Protocol::Closed
-    }
-}
-impl TryFrom<u8> for Protocol {
-    type Error = u8;
-    fn try_from(val: u8) -> Result<Protocol, u8> {
+impl Protocol {
+    /// Convert a raw `u8` to an `Protocol`.
+    ///
+    /// Bit values that do not correspond to a protocol will be returned in the
+    /// `Err` variant of the result.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use w5500_ll::Protocol;
+    ///
+    /// assert_eq!(Protocol::from_raw(0b0000), Ok(Protocol::Closed));
+    /// assert_eq!(Protocol::from_raw(0b0001), Ok(Protocol::Tcp));
+    /// assert_eq!(Protocol::from_raw(0b0010), Ok(Protocol::Udp));
+    /// assert_eq!(Protocol::from_raw(0b0100), Ok(Protocol::Macraw));
+    /// assert_eq!(Protocol::from_raw(0b0101), Err(0b0101));
+    /// ```
+    pub const fn from_raw(val: u8) -> Result<Self, u8> {
         match val {
             x if x == Protocol::Closed as u8 => Ok(Protocol::Closed),
             x if x == Protocol::Tcp as u8 => Ok(Protocol::Tcp),
@@ -379,6 +403,22 @@ impl TryFrom<u8> for Protocol {
         }
     }
 }
+impl From<Protocol> for u8 {
+    fn from(val: Protocol) -> u8 {
+        val as u8
+    }
+}
+impl Default for Protocol {
+    fn default() -> Self {
+        Self::Closed
+    }
+}
+impl TryFrom<u8> for Protocol {
+    type Error = u8;
+    fn try_from(val: u8) -> Result<Self, u8> {
+        Self::from_raw(val)
+    }
+}
 
 /// PHY operation mode.
 ///
@@ -387,6 +427,7 @@ impl TryFrom<u8> for Protocol {
 /// [`PhyCfg::opmdc`]: crate::PhyCfg::opmdc
 /// [`phycfgr`]: crate::Registers::phycfgr
 #[derive(Copy, Clone, Eq, PartialEq, Ord, PartialOrd, Debug, Hash)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 #[repr(u8)]
 pub enum OperationMode {
     /// 10BT half-duplex. Auto-negotiation disabled.
@@ -404,31 +445,60 @@ pub enum OperationMode {
     /// All capable. Auto-negotiation enabled.
     Auto = 0b111,
 }
+impl OperationMode {
+    /// Convert a raw `u8` to an `OperationMode`.
+    ///
+    /// Only the first 3 bits of the `u8` value are used.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use w5500_ll::OperationMode;
+    ///
+    /// assert_eq!(
+    ///     OperationMode::from_raw(0b000),
+    ///     OperationMode::HalfDuplex10bt
+    /// );
+    /// assert_eq!(
+    ///     OperationMode::from_raw(0b001),
+    ///     OperationMode::FullDuplex10bt
+    /// );
+    /// assert_eq!(
+    ///     OperationMode::from_raw(0b010),
+    ///     OperationMode::HalfDuplex100bt
+    /// );
+    /// assert_eq!(
+    ///     OperationMode::from_raw(0b011),
+    ///     OperationMode::FullDuplex100bt
+    /// );
+    /// assert_eq!(
+    ///     OperationMode::from_raw(0b100),
+    ///     OperationMode::HalfDuplex100btAuto
+    /// );
+    /// assert_eq!(OperationMode::from_raw(0b110), OperationMode::PowerDown);
+    /// assert_eq!(OperationMode::from_raw(0b111), OperationMode::Auto);
+    /// ```
+    pub const fn from_raw(val: u8) -> Self {
+        match val & 0b111 {
+            x if x == Self::HalfDuplex10bt as u8 => Self::HalfDuplex10bt,
+            x if x == Self::FullDuplex10bt as u8 => Self::FullDuplex10bt,
+            x if x == Self::HalfDuplex100bt as u8 => Self::HalfDuplex100bt,
+            x if x == Self::FullDuplex100bt as u8 => Self::FullDuplex100bt,
+            x if x == Self::HalfDuplex100btAuto as u8 => Self::HalfDuplex100btAuto,
+            x if x == Self::PowerDown as u8 => Self::PowerDown,
+            // x if x == Self::Auto as u8
+            _ => Self::Auto,
+        }
+    }
+}
 impl From<OperationMode> for u8 {
     fn from(val: OperationMode) -> u8 {
         val as u8
     }
 }
-impl TryFrom<u8> for OperationMode {
-    type Error = u8;
-    fn try_from(val: u8) -> Result<OperationMode, u8> {
-        match val {
-            x if x == OperationMode::HalfDuplex10bt as u8 => Ok(OperationMode::HalfDuplex10bt),
-            x if x == OperationMode::FullDuplex10bt as u8 => Ok(OperationMode::FullDuplex10bt),
-            x if x == OperationMode::HalfDuplex100bt as u8 => Ok(OperationMode::HalfDuplex100bt),
-            x if x == OperationMode::FullDuplex100bt as u8 => Ok(OperationMode::FullDuplex100bt),
-            x if x == OperationMode::HalfDuplex100btAuto as u8 => {
-                Ok(OperationMode::HalfDuplex100btAuto)
-            }
-            x if x == OperationMode::PowerDown as u8 => Ok(OperationMode::PowerDown),
-            x if x == OperationMode::Auto as u8 => Ok(OperationMode::Auto),
-            _ => Err(val),
-        }
-    }
-}
 impl Default for OperationMode {
-    fn default() -> OperationMode {
-        OperationMode::Auto
+    fn default() -> Self {
+        Self::Auto
     }
 }
 
@@ -439,6 +509,7 @@ impl Default for OperationMode {
 /// [`PhyCfg::lnk`]: crate::PhyCfg::lnk
 /// [`phycfgr`]: crate::Registers::phycfgr
 #[derive(Copy, Clone, Eq, PartialEq, Debug, PartialOrd, Ord, Hash)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 #[repr(u8)]
 pub enum LinkStatus {
     /// PHY link down.
@@ -473,6 +544,7 @@ impl Default for LinkStatus {
 /// [`PhyCfg::spd`]: crate::PhyCfg::spd
 /// [`phycfgr`]: crate::Registers::phycfgr
 #[derive(Copy, Clone, Eq, PartialEq, Debug, PartialOrd, Ord, Hash)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 #[repr(u8)]
 pub enum SpeedStatus {
     /// 10 Mbps.
@@ -507,6 +579,7 @@ impl Default for SpeedStatus {
 /// [`PhyCfg::dpx`]: crate::PhyCfg::dpx
 /// [`phycfgr`]: crate::Registers::phycfgr
 #[derive(Copy, Clone, Eq, PartialEq, Debug, PartialOrd, Ord, Hash)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 #[repr(u8)]
 pub enum DuplexStatus {
     /// Half duplex
@@ -542,6 +615,7 @@ impl Default for DuplexStatus {
 /// [`Registers::set_sn_txbuf_size`]: crate::Registers::set_sn_txbuf_size
 /// [`Registers::set_sn_rxbuf_size`]: crate::Registers::set_sn_rxbuf_size
 #[derive(Copy, Clone, Eq, PartialEq, PartialOrd, Ord, Hash, Debug)]
+#[cfg_attr(feature = "defmt", derive(defmt::Format))]
 #[repr(u8)]
 #[allow(clippy::upper_case_acronyms)]
 pub enum BufferSize {
@@ -586,7 +660,6 @@ impl TryFrom<u8> for BufferSize {
     /// # Example
     ///
     /// ```
-    /// use core::convert::TryFrom;
     /// use w5500_ll::BufferSize;
     ///
     /// assert_eq!(BufferSize::try_from(0), Ok(BufferSize::KB0));
